package lk.sliit.research.interviewapp.controller;

import lk.sliit.research.interviewapp.Service.InterviewService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.ModelAndView;

import static lk.sliit.research.interviewapp.util.Constant.APP_NAME;
import static lk.sliit.research.interviewapp.util.Constant.ERROR_LOG;

@Slf4j
@RestController
@RequestMapping("interview")
public class InterviewController {

    @Autowired
    InterviewService interviewService;

    @GetMapping("enterKeyView")
    public ModelAndView enterKeyView(){
        ModelAndView modelAndView = new ModelAndView();

        try{
            modelAndView.setViewName("interview/enterKeyView");
            modelAndView.addObject("text", APP_NAME);
        }
        catch (Exception e){
            modelAndView.setViewName("error/page");
            modelAndView.addObject("error", e.getMessage());
            log.error(ERROR_LOG, e);
        }
        return modelAndView;
    }

    @PostMapping("enterInterview")
    public ModelAndView enterInterview(@RequestParam("key") String key){
        log.info("key: {}", key);
        ModelAndView modelAndView = new ModelAndView();

        try{
            boolean isValidKey = interviewService.isValidInterviewKey(key);

            if(isValidKey){
                modelAndView.setViewName("interview/interviewPannel");
            }
            else{
                modelAndView.setViewName("interview/enterKeyView");
                modelAndView.addObject("error", "The key you entered is invalid");
            }

            modelAndView.addObject("text", APP_NAME);
        }
        catch (Exception e){
            modelAndView.setViewName("error/page");
            modelAndView.addObject("error", e.getMessage());
            log.error(ERROR_LOG, e);
        }
        return modelAndView;
    }
}
