package lk.sliit.research.interviewapp.controller;

import lk.sliit.research.interviewapp.Service.CandidateService;
import lk.sliit.research.interviewapp.model.domain.Candidate;
import lk.sliit.research.interviewapp.model.domain.Job;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.ModelAndView;

import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.Random;

import static lk.sliit.research.interviewapp.util.Constant.APP_NAME;
import static lk.sliit.research.interviewapp.util.Constant.ERROR_LOG;

@Slf4j
@RestController
@RequestMapping("candidate")
public class CandidateController {

    private static String UPLOADED_FOLDER = "C://Users//Zincat-User//Documents//Clients//Kaweesha//Interview-App-Git//interview-app//src//main//webapp//view//uploads//resume//";

    @Autowired
    CandidateService candidateService;

    @GetMapping("addCandidateView/{jobId}")
    public ModelAndView addCandidateView(@PathVariable int jobId){
        ModelAndView modelAndView = new ModelAndView();

        try{
            modelAndView.setViewName("candidate/addCandidateView");
            modelAndView.addObject("text", APP_NAME);
            modelAndView.addObject("jobId", jobId);
        }
        catch (Exception e){
            modelAndView.setViewName("error/page");
            modelAndView.addObject("error", e.getMessage());
            log.error(ERROR_LOG, e);
        }
        return modelAndView;
    }

    @PostMapping("createCandidate")
    public boolean createCandidate(@RequestBody Candidate candidate){
        log.info("Request: {}", candidate);

        try{
            return candidateService.createCandidate(candidate);
        }
        catch (Exception e){
            log.error(ERROR_LOG, e);
            return false;
        }
    }

    @GetMapping("viewAllCandidates/{jobId}")
    public ModelAndView viewAllCandidates(@PathVariable int jobId){
        ModelAndView modelAndView = new ModelAndView();

        try{
            modelAndView.addObject("candidateList", candidateService.viewAllCandidatesByJobId(jobId));
            modelAndView.setViewName("candidate/listAllCandidates");
            modelAndView.addObject("text", APP_NAME);
            modelAndView.addObject("jobId", jobId);
        }
        catch (Exception e){
            modelAndView.setViewName("error/page");
            modelAndView.addObject("error", e.getMessage());
            log.error(ERROR_LOG, e);
        }
        return modelAndView;
    }

    @GetMapping("viewUpdateResumeView/{candidateId}")
    public ModelAndView viewUpdateResumeView(@PathVariable int candidateId){
        ModelAndView modelAndView = new ModelAndView();

        try{
            Candidate candidate = candidateService.getCandidateById(candidateId);

            modelAndView.setViewName("candidate/viewUpdateCandidateView");
            if(candidate.getFile() != null){
                modelAndView.addObject("file", candidate.getFile());
            }
            modelAndView.addObject("candidateId", candidateId);
        }
        catch (Exception e){
            modelAndView.setViewName("error/page");
            modelAndView.addObject("error", e.getMessage());
            log.error(ERROR_LOG, e);
        }
        return modelAndView;
    }

        @PostMapping("uploadCandidateResume")
    public ModelAndView uploadCandidateResume(@RequestParam("file") MultipartFile file, @RequestParam("candidateId") int candidateId){
        log.info("candidateId: {}", candidateId);

        ModelAndView modelAndView = new ModelAndView();
        try {
            Candidate candidate = candidateService.getCandidateById(candidateId);

            if (file.isEmpty()) {
                log.info("message: Please select a file to upload");
                modelAndView.addObject("file", candidate.getFile());
                modelAndView.setViewName("candidate/viewUpdateCandidateView");
                modelAndView.addObject("candidateId", candidateId);
                modelAndView.addObject("uploadCandidateResumeMessage", "Please select a file to upload!!!");

                return modelAndView;
            }

            /**
             * Restrict pdf documents
             * */
            String fileExtension = file.getOriginalFilename().split("\\.")[1];
            if(!fileExtension.equalsIgnoreCase("pdf")){
                log.info("message: You can only upload pdf files");
                modelAndView.setViewName("candidate/viewUpdateCandidateView");
                modelAndView.addObject("candidateId", candidateId);
                modelAndView.addObject("uploadCandidateResumeMessage", "You can only upload pdf files!!!");

                return modelAndView;
            }

            int leftLimit = 97; /** letter 'a' */
            int rightLimit = 122; /** letter 'z' */
            int targetStringLength = 6;
            Random random = new Random();
            StringBuilder buffer = new StringBuilder(targetStringLength);
            for (int i = 0; i < targetStringLength; i++) {
                int randomLimitedInt = leftLimit + (int)
                        (random.nextFloat() * (rightLimit - leftLimit + 1));
                buffer.append((char) randomLimitedInt);
            }
            String generatedString = buffer.toString();

            /** Get the file and save it somewhere */
            byte[] bytes = file.getBytes();
            String file_name = generatedString + "_" + file.getOriginalFilename();
            Path path = Paths.get(UPLOADED_FOLDER + file_name);
            Files.write(path, bytes);

            log.info("File name: " + file_name);

            candidate.setFile(file_name);
            candidateService.updateCandidate(candidate);

            return viewAllCandidates(candidate.getJobId());
        }
        catch (Exception e){
            log.error(ERROR_LOG, e);
            modelAndView.setViewName("candidate/viewUpdateCandidateView");
            modelAndView.addObject("candidateId", candidateId);
            modelAndView.addObject("uploadCandidateResumeMessage", "Error: " + e.getMessage());
        }

        return modelAndView;
    }

}
