package lk.sliit.research.interviewapp.controller;

import lk.sliit.research.interviewapp.Service.InterviewService;
import lk.sliit.research.interviewapp.Service.QnaInterviewService;
import lk.sliit.research.interviewapp.model.domain.Interview;
import lk.sliit.research.interviewapp.model.dto.request.GetQuestionRequestDto;
import lk.sliit.research.interviewapp.model.dto.response.GetQuestionResponseDto;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.ModelAndView;

import java.util.ArrayList;
import java.util.List;

import static lk.sliit.research.interviewapp.util.Constant.*;

@Slf4j
@RestController
@RequestMapping("interview")
public class InterviewController {

    @Autowired
    InterviewService interviewService;

    @Autowired
    QnaInterviewService qnaInterviewService;

    @GetMapping("enterKeyView")
    public ModelAndView enterKeyView(){
        ModelAndView modelAndView = new ModelAndView();

        try{
            modelAndView.setViewName("interview/enterKeyView");
            modelAndView.addObject("text", APP_NAME);
        }
        catch (Exception e){
            modelAndView.setViewName("error/page");
            modelAndView.addObject("error", e.getMessage());
            log.error(ERROR_LOG, e);
        }
        return modelAndView;
    }

    @PostMapping("enterInterview")
    public ModelAndView enterInterview(@RequestParam("roomId") String roomId, @RequestParam("key") String key){
        log.info("key: {}", key);
        ModelAndView modelAndView = new ModelAndView();

        try{
            Interview interview = interviewService.getInterviewByRoomIdAndKey(roomId, key);
            String question = "Hello candidate, welcome to the interview. I will be doing your interview. What is your name?";

            if(interview != null){
                interviewService.updateInterviewLifeCycle(interview.getId(), INTERVIEW_STATUS_PROCESSING);

                modelAndView.setViewName("interview/interviewPannel");
                modelAndView.addObject("question", question);
                modelAndView.addObject("interviewId", interview.getId());
            }
            else{
                modelAndView.setViewName("interview/enterKeyView");
                modelAndView.addObject("error", "The key you entered is invalid, or the interview is not placed. Please contact ADMIN for more information");
            }

            modelAndView.addObject("text", APP_NAME);
        }
        catch (Exception e){
            modelAndView.setViewName("error/page");
            modelAndView.addObject("error", e.getMessage());
            log.error(ERROR_LOG, e);
        }
        return modelAndView;
    }

    @PostMapping("getQuestion")
    public GetQuestionResponseDto getQuestion(@RequestBody GetQuestionRequestDto getQuestionRequestDto){
        GetQuestionResponseDto getQuestionResponseDto = new GetQuestionResponseDto();
        try{
            log.info("GetQuestionDto: {}", getQuestionRequestDto);

            qnaInterviewService.addInterviewAnswer(getQuestionRequestDto.getInterViewId(), getQuestionRequestDto.getPreviousQuestionId(), getQuestionRequestDto.getAnswer());

            if(isEndOfQuestion(getQuestionRequestDto)){
                interviewService.updateInterviewLifeCycle(getQuestionRequestDto.getInterViewId(), INTERVIEW_STATUS_INTERVIEW_OVER);

                getQuestionResponseDto.setQuestionId(-1);
                getQuestionResponseDto.setQuestion("end");

                return getQuestionResponseDto;
            }

            getQuestionResponseDto.setQuestionId(getQuestionRequestDto.getCount() + 1);
            getQuestionResponseDto.setQuestion(getMockQuestion(getQuestionRequestDto.getCount()));
            return getQuestionResponseDto;
        }
        catch (Exception e){
            log.error(ERROR_LOG, e);

            getQuestionResponseDto.setQuestionId(0);
            getQuestionResponseDto.setQuestion(ERROR);
            return getQuestionResponseDto;
        }
    }

    /**
     * Function to decide end of question
     * Restriction for now: Restricted after question count is 10
     * */
    private boolean isEndOfQuestion(GetQuestionRequestDto getQuestionRequestDto){
        return getQuestionRequestDto.getCount() >= 10;
    }

    private String getMockQuestion(int id){
        List<String> mockQuestionList = new ArrayList<>();

        mockQuestionList.add("Explain some of the SDLC models.");
        mockQuestionList.add("Can you explain what verification is in software development?");
        mockQuestionList.add("Can you explain what validation is in software development?");
        mockQuestionList.add("Define the elements you should consider in the System Model Construction.");
        mockQuestionList.add("What are the main tools you use to monitor the software’s requirements?");
        mockQuestionList.add("How are stubs different from mocks?");
        mockQuestionList.add("Which SDLC model would you say is superior to others?");
        mockQuestionList.add("How would you describe what software scope means?");
        mockQuestionList.add("How would you find out how big a software product is?");
        mockQuestionList.add("How would you define function points?");

        return mockQuestionList.get(id);
    }
}
